package com.zykj.temperaturedemo;

import android.util.Log;

import java.io.File;
import java.io.IOException;

import android_serialport_api.SerialPort;

public class TempDataReceiveTTL extends TTLReceiveBase{
//    private byte[] queryBodyTemp = new byte[]{(byte)0xA5,(byte)0x55,(byte)0x01,(byte)0xFB};
    private byte[] queryTemp = new byte[]{(byte)0x43,(byte)0x61,(byte)0x6C,(byte)0x63,(byte)0x54,(byte)0x0D,(byte)0x0A};
    private int iBaudRate = 115200;
    private String port = "/dev/ttyS3";

    public boolean open() {
        try {
            mSerialPort = new SerialPort(new File(port), iBaudRate, 0);
            input = mSerialPort.getInputStream();
            output = mSerialPort.getOutputStream();

            byte[] data = new byte[10];
            float temp = getBodyTempData(data);
            if(temp>0){
                return true;
            } else {
                close();
            }
        } catch (Exception e) {
            e.printStackTrace();
            close();
        }
        return false;
    }

    /**
     * 读取一帧体温值
     * @param data new byte[10]
     * @return 体温值
     * */
    public float getBodyTempData(byte[] data) {
        skipCacheData();
        try {
            output.write(queryTemp);
        } catch (IOException e) {
            Log.i("TEMP","发送 查询输出数据指令 失败");
            return -1;
        }

        long startTime = System.currentTimeMillis();
        long overTime = 1000;
        //等待直到available 或 超时
        int length = 0;
        try {
            length = input.available();
            while(length == 0 && Math.abs(System.currentTimeMillis()-startTime)<overTime){
                length = input.available();
                if(length>0) break;
                try { Thread.sleep(10); } catch (InterruptedException e) {}
            }
        } catch (IOException e) {
            return -1;
        }
        if(length==0) return -1;

        try {
            int size = input.read(data,0,data.length);
            if(size == data.length && data[0] == 0x43 && data[1] == 0x54){
                //CRC16校验
                int crc16 = CRC16Util.calcCrc16(data,2,data.length-4);
                int check = ((data[data.length-2]&0xff) << 8 | (data[data.length-1]&0xff)) & 0xffff ;
                if(crc16 == check){
                    //计算温度：
                    return (float)(data[2]+(data[3]/100.0f));
                } else {
                    Log.e("TEMP","校验和错误");
                }
            }
        } catch (IOException e) {
            Log.e("TEMP","发送 读取配置回复错误",e);
        }
        return -1;
    }

    public void destroy() {
        close();
    }
}
